import sys

from colorama import Fore, Style, Back

from galoup.error import Error


def normal(line: str, end='\n'):
    print(line, file=sys.stderr, end=end, flush=True)


def bad(line: str):
    print(Fore.RED + line + Style.RESET_ALL, file=sys.stderr, flush=True)


def good(line: str):
    print(Fore.GREEN + Style.BRIGHT + line + Style.RESET_ALL, file=sys.stderr, flush=True)


def error(e: Error):
    print(Style.BRIGHT + Fore.LIGHTWHITE_EX + Back.RED + type(e).__name__ + ":"
          + Style.NORMAL + Back.RESET + Fore.RESET + " " + e.message + Style.RESET_ALL,
          file=sys.stderr, flush=True)


def warning(line: str):
    print(Fore.YELLOW + Style.BRIGHT + line + Style.RESET_ALL, file=sys.stderr, flush=True)



class Error(Exception):
    def __init__(self, message):
        self.message = message


class ValidationError(Error):
    def __init__(self, message):
        super().__init__(message)


class ProcessError(Error):
    def __init__(self, message):
        super().__init__(message)


class NotYetImplementedError(Error):
    def __init__(self, message):
        super().__init__(message)



import subprocess


def run(cmd: str):
    process = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, error = process.communicate()
    return output.decode("utf-8"), error.decode("utf-8")



import glob
import os
import pathlib
import re


def relative_to_project_folder(path: str, project_folder: str) -> str:
    return str(pathlib.Path(os.path.realpath(path)).relative_to(os.path.join(os.getcwd(), project_folder)))


def mkdirp(path: str):
    pathlib.Path(path).mkdir(parents=True, exist_ok=True)


def replace_in_files(file_pattern: str, regex_match: str, repl: str):
    paths = glob.glob(file_pattern)
    for location in paths:
        # Read whole file
        contents = pathlib.Path(location).read_text()
        # Replace
        new_contents = re.sub(regex_match, repl, contents)
        # Replace file
        pathlib.Path(location).write_text(new_contents)



import argparse
import os
import sys
from typing import List, Callable, Dict, Any

import colorama

from galoup import logger, process, error
from galoup.error import Error

ParserAssignmentFunction = Callable[[argparse.ArgumentParser], None]
OperationFunction = Callable[[Dict[Any, Any]], None]


class Operation:
    def __init__(self, description: str, operation_func: OperationFunction):
        self.description = description
        self.operation_func = operation_func

    def run(self, current_vars: Dict[Any, Any]):
        logger.normal(self.description + "... ", end='')
        try:
            self.operation_func(current_vars)
            logger.good("Done!")
        except Error as e:
            logger.bad("Failed.")
            raise e


class Command:
    def __init__(self, name: str, description: str, parser_func: ParserAssignmentFunction, operations: List[Operation]):
        self.name = name
        self.description = description
        self.parser_func = parser_func
        self.operations = operations

    def register(self, parent_parser: argparse._SubParsersAction):
        subparser = parent_parser.add_parser(name=self.name, description=self.description)
        subparser.set_defaults(func=self.run)
        self.parser_func(subparser)

    def run(self, opts: argparse.Namespace):
        current_vars = {}
        current_vars.update(opts.__dict__)
        for op in self.operations:
            op.run(current_vars)


class Program:
    def __init__(self, name: str, description: str, commands: List[Command]):
        self.name = name
        self.description = description
        self.commands = commands

    def run(self):
        # Get args
        parser = argparse.ArgumentParser(prog=self.name, description=self.description)
        subparsers = parser.add_subparsers(title='sub-commands')
        for command in self.commands:
            command.register(subparsers)
        # Init global fixtures
        colorama.init()
        # Do some basic validation
        try:
            _validate()
        except Error as e:
            _fail(e)
        # Get args
        parsed = parser.parse_args(sys.argv[1:])
        if not hasattr(parsed, 'func'):
            # If we get here, it means a sub-command wasn't given
            logger.normal("You must specify a sub-command.")
            parser.print_usage(sys.stderr)
            sys.exit(1)
        # Execute sub-command
        try:
            parsed.func(parsed)
        except Error as e:
            _fail(e)


def _in_project_root() -> bool:
    # Get the git root directory
    output, _ = process.run('git rev-parse --show-toplevel')
    # Get the current working directory
    cwd = os.getcwd()
    # See if they match
    return os.path.realpath(output.rstrip()) == os.path.realpath(cwd)


def _validate():
    if not _in_project_root():
        raise error.ValidationError("You must be in the project root directory to run galoup.")


def _fail(e: Error):
    logger.error(e)
    sys.exit(1)



from galoup import process
from galoup.error import ProcessError, NotYetImplementedError


def get_origin():
    output, error = process.run('git config --get remote.origin.url')
    if len(error) != 0:
        raise ProcessError("while trying to get git origin: " + error)
    output = output.rstrip()
    # SSH style
    if "@" in output:
        without_user = output.split("@", 1)[1]
        colon_split = without_user.split(':', 1)
        repo_provider, without_repo_provider = colon_split[0], "".join(colon_split[1:])
        repo_part = without_repo_provider.rsplit(".", 1)[0]
        return repo_provider, repo_part
    else:
        raise NotYetImplementedError("cannot handle non SSH git origins. Please implement to continue.")




from setuptools import setup

distributed_version = '0.0.4-alpha'

setup(
    name='galoup',
    version=distributed_version,
    packages=['galoup'],
    url='https://github.com/liampulles/galoup',
    license='MIT',
    author='Liam Pulles',
    author_email='me@liampulles.com',
    description='Galoup is a Python framework for rolling your own project management tool.',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.4',
    install_requires=[
        "colorama"
    ]
)
